
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;
/**
 *Clase de tipo Equipment que implementa los graficos desplegados
 *para cada estanque
 *
 * @author Daniel Zuleta
 * @author Mauricio Diaz
 */

public class grafico extends Equipment{
    
    private int ancho = 300;
    private int alto = 105;
    private int x;
    private int y,y1,y2;
    private Tank tanke;
    private Controller c;
    private int i, tipo;
    private ArrayList aY1 = new ArrayList();
    private ArrayList aY2 = new ArrayList();
    private ArrayList aY3 = new ArrayList();
    private JPanel gPanel1, gPanel2;
    
	  /**
     * Contructor 1 de la clase, que implementa 2 mediciones
	  * Nivel del estanque respectivo y Porcentaje de apertura
	  * de la valvula
     * @param x Componente x del origen del grafico 
	  * @param y Componente y del origen del grafico
	  * @param t Referencia al estanque que se quiere graficar
     */
    public grafico(int x, int y, Tank t) {
	
        this.x=x;
        this.y=y;
        aY1.add(y);
        tanke = t;
        gPanel1 = this.crearEje(x-16, y-96);
        gPanel2 = this.leyenda(x+ancho-30, y-100, 2);
        tipo = 1;
    }
	 /**
     * Contructor  de la clase, que implementa 2 mediciones
	  * Nivel del estanque respectivo, Porcentaje de apertura
	  * de la valvula y Referencia
     * @param x Componente x del origen del grafico 
	  * @param y Componente y del origen del grafico
	  * @param t Referencia al estanque que se quiere graficar
	  * @param c Referencia al controlador para tomar el nivel de referencia
     */
    public grafico(int x, int y, Tank t, Controller c) {
	
        this.x=x;
        this.y=y;
        aY1.add(y);
        tanke = t;
        this.c=c;
        gPanel1 = this.crearEje(x-16, y-96);
        gPanel2 = this.leyenda(x+ancho-30, y-100, 3);
        tipo = 2;
    }
     /**
     *Metodo implementado segun la clase abstracta Equipment
	  *para llamar a repintado de las componentes
     */
    public void paint(Graphics2D g2d){
        
        g2d.setColor(Color.black);
        g2d.drawLine(x, y, x, y-alto);
        g2d.drawLine(x, y, x+ancho, y);
        
        aY1.add(new Integer(y-(int)tanke.getLevel()));
        aY2.add(new Integer(y-(int)(tanke.getOutValve()*100)));
                
        for(i=0;i<aY1.size()-1;i++){
            y1=Integer.parseInt(aY1.get(i).toString());
            y2=Integer.parseInt(aY1.get(i+1).toString());
            g2d.drawLine(x+i,y1,x+i+1,y2);
        }
        if(aY1.size()>=300)aY1.remove(0);
        
        g2d.setColor(Color.red);
        for(i=0;i<aY2.size()-1;i++){
            y1=Integer.parseInt(aY2.get(i).toString());
            y2=Integer.parseInt(aY2.get(i+1).toString());
            g2d.drawLine(x+i,y1,x+i+1,y2);
        }
        if(aY2.size()>=300)aY2.remove(0);
        
        g2d.setColor(Color.blue);
        if(tipo==2){
            aY3.add(new Integer(y-(int)c.getRef()));
            for(i=0;i<aY3.size()-1;i++){
                y1=Integer.parseInt(aY3.get(i).toString());
                y2=Integer.parseInt(aY3.get(i+1).toString());
                g2d.drawLine(x+i,y1,x+i+1,y2);
            }
            if(aY3.size()>=300)aY3.remove(0);
        }
    }
 
    public void timeTick(float dt){

    }
    
    public Box getGUI(){
	Box gui = new Box(BoxLayout.X_AXIS);
	return gui;
    }
     /** 
     * Crea las cifras del eje y 
	  * @param posX Posicion con respecto al eje x donde se insertaran las cifras
	  * @param posY Posicion con respecto al eje y donde se insertaran las cifras
	  * @return El JPanel que contiene los elementos a insertar en la aplicacion
     */

    private JPanel crearEje(int posX, int posY){
       
       JPanel menu = new JPanel();
	menu.setBounds(posX,posY,15,130);
	menu.setLayout(new GridLayout(22,1));
	
	int j = 10;
	for(int i=0; i<22; i++){
	    
	    int num = 10*j;
	    if(i%2 == 0){ 
		
		JLabel aux = new JLabel ("" + num,SwingConstants.RIGHT);
		aux.setFont(new Font("Plain", Font.PLAIN, 7)); 
		menu.add(aux);
	        j--;
	  }else {                  
	      
	      
	      JLabel aux = new JLabel("");
	      aux.setFont(new Font("Italic", Font.ITALIC, 7)); 
	      menu.add(aux);
	     
	   
	  }
	    
	}
	return menu;
   }
    /**
	 * return Retorna una JPanel para graficos de 2 mediciones
	 */
    public JPanel getPanel1(){
        return gPanel1;
    }
    /**
	 * return Retorna una JPanel para graficos de 3 mediciones
	 */
    public JPanel getPanel2(){
        return gPanel2;
    }
    /**
	 * Metodo implementado para crear una leyenda para cada tipo de grafico
	 * @param posX Posicion con respecto al eje x donde se insertara la leyenda
	 * @param posY Posicion con respecto al eje y donde se insertara la leyenda
	 * @param numLineas Parametro que indica el numero de mediciones que tiene el grafico
	 * @return El JPanel que contiene la leyenda a insertar en la aplicacion
	 */
    public JPanel leyenda(int posX, int posY, int numLineas){
        JPanel legend = new JPanel();
	legend.setBounds(posX,posY,80,30);
	legend.setLayout(new GridLayout(numLineas,1));
        
        JLabel aux = new JLabel ("% Nivel Estanque");
        aux.setFont(new Font("Plain", Font.PLAIN, 9));
        aux.setForeground(Color.black);
	legend.add(aux);
        
        
        JLabel aux2 = new JLabel ("% Apertura Valvula");
        aux2.setFont(new Font("Plain", Font.PLAIN, 8)); 
        aux2.setForeground(Color.red);
	legend.add(aux2);
        
        if(numLineas==3){
            JLabel aux3 = new JLabel ("Nivel Referencia");
            aux3.setFont(new Font("Plain", Font.PLAIN, 9)); 
            aux3.setForeground(Color.blue);
            legend.add(aux3);
        }
        
        return legend;
    }
}
